package cn.com.ty.lift.business.maintenance.service;

import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceDetailRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceGenerateRequest;
import cn.com.ty.lift.business.project.dao.entity.ProjectLiftRelevance;
import cn.com.ty.lift.business.project.service.ProjectLiftRelevanceService;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenancePlan;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenancePlanRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MaintenancePlanResponse;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceModifyRequest;
import cn.com.ty.lift.business.maintenance.dao.mapper.MaintenancePlanMapper;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.service.ProjectService;
import cn.com.ty.lift.common.base.ExportRequest;
import cn.com.ty.lift.common.export.ExportUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.*;

/**
 * @author bieao
 * @date 2019/12/14
 * @description 维保计划业务层
 */
@Service
@Slf4j
public class MaintenancePlanService extends ServiceImpl<MaintenancePlanMapper, MaintenancePlan> {

    @Resource
    private ProjectService projectService;

    @Resource
    private MaintenancePlanMapper planMapper;

    @Resource
    private ProjectLiftRelevanceService projectLiftRelevanceService;


    private Map<String, String> paramMap = new HashMap<String, String>() {{
        put("projectCode", "项目编号");
        put("projectName", "项目名称");
        put("regionName", "区域名称");
        put("registrationCode", "注册代码");
        put("devicePosition", "电梯位置");
        put("workerName", "维保工");
        put("", "间隔");
        put("liftStatus", "电梯维保状态");
        put("planDate", "计划时间");
    }};

    /**
     * @param request 维保计划列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 查询维保计划
     * @date 2019/12/24 10:47 AM
     */
    public IPage<MaintenancePlanResponse> list(MaintenancePlanRequest request) {
        IPage<MaintenancePlanResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return planMapper.findByCondition(page, request);
    }

    /**
     * @param request 公司id，项目id，电梯id
     * @return 1.成功, 0.失败, 消息描述
     * @description 查询维保计划详情列表
     * @date 2020/1/2 9:43 上午
     */
    public List<MaintenancePlan> detailList(MaintenanceDetailRequest request) {
        Map<String, Object> paramMap = new HashMap<>(3);
        paramMap.put("mt_company_id", request.getMtCompanyId());
        paramMap.put("project_id", request.getProjectId());
        paramMap.put("lift_id", request.getLiftId());
        return planMapper.selectByMap(paramMap);
    }

    /**
     * @param request 电梯列表
     * @return 是否成功
     * @description 批量生成维保计划
     * @date 2019/12/16 2:14 PM
     */
    public boolean insertBatch(MaintenanceGenerateRequest request) {
        List<MaintenancePlan> resultList = generatePlan(request);
        return saveBatch(resultList, resultList.size());
    }

    /**
     * @param beginTime 项目开始时间
     * @param endTime   项目结束时间
     * @param interval  保养间隔
     * @return 实际期数
     * @description 计算当前电梯所在项目开始时间到结束时间需要保养的期数
     * @date 2019/12/16 2:11 PM
     */
    public int calculatePeriods(LocalDate beginTime, LocalDate endTime, int interval) {
        long days = beginTime.until(endTime, ChronoUnit.DAYS);
        double num = Math.floor(Double.parseDouble(String.valueOf((days / interval))));
        return (int) num;
    }

    /**
     * @param request 需要保养的电梯
     * @return List<MaintenancePlan> 维保计划列表
     * @description 生成维保计划
     * @date 2019/12/16 1:21 PM
     */
    public List<MaintenancePlan> generatePlan(MaintenanceGenerateRequest request) {
        List<MaintenancePlan> plans = request.getPlanList();
        //返回维保计划列表
        List<MaintenancePlan> planList = new ArrayList<>();
        //保养类型 1.半月，2.季度，3.半年，4.全年
        int[] maintenanceType = {1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 3, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 4};
        int times = 0;
        //获取从第几期开始
        int periods = request.getPeriods();
        //获取保养间隔
        int interval = request.getInterval();
        //获取首保时间
        LocalDate firstTime = request.getFirstTime();
        for (MaintenancePlan plan : plans) {
            //获取项目id
            Long projectId = plan.getProjectId();
            Project project = projectService.detail(projectId);
            if (project != null) {
                LocalDate beginTime = project.getStartDate();
                LocalDate endTime = project.getEndDate();
                //计算当前电梯所在项目开始时间到结束时间需要保养的期数
                times = calculatePeriods(beginTime, endTime, interval);
            }
            for (int i = 0; i < times; i++) {
                MaintenancePlan entry = new MaintenancePlan();
                //设置维保计划电梯ID，维保公司ID，项目ID，维保工ID，维保类型
                entry.setLiftId(plan.getLiftId());
                entry.setMtCompanyId(plan.getMtCompanyId());
                entry.setProjectId(projectId);
                entry.setWorkerId(plan.getWorkerId());
                entry.setType(maintenanceType[periods - 1]);
                if (i == 0) {
                    entry.setPlanDate(firstTime);
                } else {
                    //设置维护保养时间
                    MaintenancePlan maintenancePlan = planList.get(i - 1);
                    LocalDate planDate = maintenancePlan.getPlanDate().plusDays(interval);
                    entry.setPlanDate(planDate);
                }
                System.out.printf("i：%s,期数：%s,类型：%s,计划时间：%s,%s", (i + 1), periods, entry.getType(), entry.getPlanDate(), "\n");
                periods++;
                if (periods == 25) {
                    periods = 1;
                }
                planList.add(entry);
            }
        }
        //更新项目电梯关联表中电梯的首保时间和保养间隔
        List<ProjectLiftRelevance> entryList = request.getRelevanceList();
        entryList.forEach(entry -> {
            entry.setPlanInterval(interval);
            entry.setFirstTime(firstTime);
        });
        projectLiftRelevanceService.updateBatchById(entryList);
        return planList;
    }

    /**
     * @param request 修改维保计划请求
     * @return 是否修改成功
     * @description 修改维保计划
     * @date 2019/12/16 5:25 PM
     */
    public boolean modifyPlan(MaintenanceModifyRequest request) {
        List<MaintenancePlan> resultList = new ArrayList<>();
        //获取当前保养时间
        LocalDate currentTime = request.getCurrentTime();
        //获取修改的保养时间
        LocalDate updateTime = request.getUpdateTime();
        //获取偏移量
        long offset = currentTime.until(updateTime, ChronoUnit.DAYS);
        //获取当前时间后的维保计划id列表
        List<MaintenancePlan> idList = planMapper.findIdList(request);
        idList.forEach(entry -> {
            LocalDate planDate = entry.getPlanDate().plusDays(offset);
            entry.setPlanDate(planDate);
            resultList.add(entry);
        });
        return updateBatchById(resultList, resultList.size());
    }

    /**
     * @param idList 维保计划主键id列表
     * @return 是否成功
     * @description 批量清除维保计划
     * @date 2019/12/28 2:00 PM
     */
    public boolean batchCleanPlan(List<Long> idList) {
        return removeByIds(idList);
    }


    /**
     * @param companyId 公司id
     * @param idList    电梯id列表
     * @return 是否成功
     * @description 批量清除维保计划
     * @date 2019/12/28 2:00 PM
     */
    public boolean removeMaintenancePlan(String companyId, List<Long> idList) {
        QueryWrapper<MaintenancePlan> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("mt_company_id", companyId);
        queryWrapper.in("lift_id", idList);
        return remove(queryWrapper);
    }

    /**
     * @param mtCompanyId 公司id
     * @param liftId      电梯id
     * @return 是否成功
     * @description 清空维保计划
     * @date 2019/12/21 3:39 PM
     */
    public boolean cleanPlan(String mtCompanyId, String liftId) {
        Map<String, Object> columnMap = new HashMap<>(2);
        columnMap.put("mt_company_id", mtCompanyId);
        columnMap.put("lift_id", liftId);
        return planMapper.deleteByMap(columnMap) > 0;
    }

    /**
     * @param request 导出项目数据列表
     * @return RestResponse 状态码和返回消息
     * @description 导出维保计划
     * @date 2019/12/19 11:27 AM
     */
    public void export(ExportRequest request) {
        List<MaintenancePlan> projectList = planMapper.selectBatchIds(request.getIds());
        ExportUtils utils = new ExportUtils();
        utils.export(projectList, paramMap, request.getExportPath());
    }
}
